<?php
/*
 * ss_ipw_stats.php v1.06
 * -----------------------------------------------------------------------------
 * version rel: v1.0 	initial
 *				v1.02	template tuning and floating point correction
 *				v1.03	mib values 
 							-inc_handover		corrected if cond for rate
 							-nb_throughput		multiplication(kbps) 
 							-nb_dl/ulresource	added totalmib
 							-nb_ulTFCS			corrected ul mib query oid
 							-nb_allocinterval	correction on filtering values
 				v1.04	added function
 							-inc_l2tp_stats		active l2tp and cells
 							-nb_uibts_stats		active ibts stats
 							-nb_numue_stats		ue / tos and total
 							-nb_ulavebitrate 
 							-nb_dlavebitrate
 				v1.05	almost final stage --added
 							-entire net inc users
 							-entire net inc throughput
				v1.06	adding standard format_snmp_string (no longer rely on cacti-lib-snmp.php)
						adding REGEXP_SNMP_TRIM
 * -----------------------------------------------------------------------------
 * enables cacti to read ipw related kpi statistics
 *
 * Originally by fendi@get-gabs.com - 2008/09/01
 *
 * usage:
 * from the command line:
 * ss_ipw_stats.php <section> inc_host nb_cellid
 *
 * as a script server object:
 * ss_ipw_stats.php ss_ipw_stats <section> inc_host nb_cellid
 *
 * Usage: ipw_stats.php <section> <inc_host> <nb_cellid>
 * Where <section> is one of: inc / nb stats
 * 		inc -> inc_access_rate, inc_cn_stats, inc_handover_stats, inc_cpu_stats, inc_l2tp_stats
 *		nb	-> nb_connresel_rate, nb_throughput_stats, nb_dlTFCS_stats, nb_ulTFCS_stats
 			   nb_queue_stats, nb_allocinterval_stats, nb_dlresource_stats, nb_ulresouce_stats
 			   nb_uibts_stats, nb_numue_stats, nb_ulavebitrate, nb_dlavebitrate
 *
*/

define("REGEXP_SNMP_TRIM", "(hex|counter(32|64)|gauge|gauge(32|64)|float|ipaddress|string|integer):");

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0]) || isset($_SERVER['REQUEST_METHOD'])  || isset($_SERVER['REMOTE_ADDR'])) {
   die("<br><strong>This script is only meant to run at the command line.</strong>");
}

$no_http_headers = true;

/* display No errors */
error_reporting(0);

include_once(dirname(__FILE__) . "/../include/global.php");
#include_once(dirname(__FILE__) . "/../lib/snmp.php");

if (!isset($called_by_script_server)) {
	array_shift($_SERVER["argv"]);
	print call_user_func_array("ss_ipw_stats", $_SERVER["argv"]);
}

function ss_ipw_stats($section, $inc_host, $nb_cellid) {
	if (empty($section) || empty($inc_host) || empty($nb_cellid)) {
		cacti_log("ERROR: ipwStats - Parameters Missing, can not continue :: $section, $inc_host, $nb_cellid");
	}
	
	switch ($section){ 
	case "incs_user":
		$output = incs_user($inc_host);
	break;

	case "incs_throughput":
		$output = incs_throughput($inc_host);
	break;
	
	case "inc_access_rate":
		$output = inc_access_rate($inc_host);
	break;

	case "inc_cn_stats":
		$output = inc_cn_stats($inc_host);
	break;

	case "inc_cpu_stats":
		$output = inc_cpu_stats($inc_host);
	break;
	
	case "inc_handover_stats":
		$output = inc_handover_stats($inc_host);
	break;
	
	case "inc_l2tp_stats":
		$output = inc_l2tp_stats($inc_host);
	break;

	case "nb_connresel_rate":
		$output = nb_connresel_rate($inc_host, $nb_cellid);
	break;
	
	case "nb_throughput_stats": //nb_throughput_stats
		$output = nb_throughput_stats($inc_host, $nb_cellid);
	break;

	case "nb_dlTFCS_stats":
		$output = nb_dlTFCS_stats($inc_host, $nb_cellid);
	break;
	
	case "nb_ulTFCS_stats":
		$output = nb_ulTFCS_stats($inc_host, $nb_cellid);
	break;

	case "nb_queue_stats":
		$output = nb_queue_stats($inc_host, $nb_cellid);
	break;
	
	case "nb_allocinterval_stats":
		$output = nb_allocinterval_stats($inc_host, $nb_cellid);
	break;
	
	case "nb_dlresource_stats":
		$output = nb_dlresource_stats($inc_host, $nb_cellid);
	break;
	
	case "nb_ulresource_stats":
		$output = nb_ulresource_stats($inc_host, $nb_cellid);
	break;
	
	case "nb_ibts_stats":
		$output = nb_ibts_stats($inc_host, $nb_cellid);
	break;
	
	case "nb_numue_stats":
		$output = nb_numue_stats($inc_host, $nb_cellid);
	break;
	
	case "nb_ulavebitrate":
		$output = nb_ulavebitrate($inc_host, $nb_cellid);
	break;
	
	case "nb_dlavebitrate":
		$output = nb_dlavebitrate($inc_host, $nb_cellid);
	break;
	
	default :
		cacti_log("ERROR: ipwStats - Unknown section request");
		return "U";
	}
	return $output;
}

// entire net inc stats procedures---------------------------------------------------------------------------------------------
function incs_user($inc_hostlist){
	/*
	coreNetworkMisc .1.3.6.1.4.1.5586.3.1.2.3.4.	1	//cnDownlinkDataRate 	(kbps)	[0]
													2	//cnDownlinkPacketRate	(pktps)	[1]
													3	//l2tpTotalActiveSessions		[2]
													4	//cnUplinkDataRate		(kbps)	[3]
													5	//cnUplinkPacketRate	(pktps)	[4]
	raStatsEntry	1.3.6.1.4.1.5586.3.2.2.2.5.3.1.
						4	raNumberOfUEsConnected
	*/
	
	$iuser_oid = ".1.3.6.1.4.1.5586.3.1.2.3.4.3";
	$iuserincip = split(",", ltrim($inc_hostlist));
	
	$iuserSum=0; 
	for ($a=0; $a < count($iuserincip); $a++) {
		if (strlen($iuserincip[$a]) > 0) {
			$iusertotal = format_snmp_string(get_ipw_snmpwalk($iuserincip[$a], $iuser_oid));
				if (is_int(intval($iusertotal))) {
					$iuserSum = $iuserSum + $iusertotal;
				}
		}
	}

	return sprintf("sumUser:%u",$iuserSum);
}

function incs_throughput($inc_hostlist){
	/*
	coreNetworkMisc .1.3.6.1.4.1.5586.3.1.2.3.4.	1	//cnDownlinkDataRate 	(kbps)	[0]
													2	//cnDownlinkPacketRate	(pktps)	[1]
													3	//l2tpTotalActiveSessions		[2]
													4	//cnUplinkDataRate		(kbps)	[3]
													5	//cnUplinkPacketRate	(pktps)	[4]
	raStatsEntry	1.3.6.1.4.1.5586.3.2.2.2.5.3.1.
						4	raNumberOfUEsConnected
	*/
	
	$idlrate_oid = ".1.3.6.1.4.1.5586.3.1.2.3.4.1";
	$iulrate_oid = ".1.3.6.1.4.1.5586.3.1.2.3.4.4";	
	$irateincip = split(",", ltrim($inc_hostlist));
	
	$idlrateSum=0; $iulrateSum=0;
	for ($a=0; $a < count($irateincip); $a++) {
		if (strlen($irateincip[$a]) > 0) {
			$idlratetotal = format_snmp_string(get_ipw_snmpwalk($irateincip[$a], $idlrate_oid));
			$iulratetotal = format_snmp_string(get_ipw_snmpwalk($irateincip[$a], $iulrate_oid));
				if (is_int(intval($idlratetotal))) {
					$idlrateSum = $idlrateSum + $idlratetotal;
				} 
				if (is_int(intval($iulratetotal))) {
					$iulrateSum = $iulrateSum + $iulratetotal;
				}
		}
	}
	
	return sprintf("sumDLrate:%u sumULrate:%u", $idlrateSum, $iulrateSum);
}


// inc stats procedures---------------------------------------------------------------------------------------------
function inc_access_rate($inc_ipaddr){
	
	$iar_oid = array(".1.3.6.1.4.1.5586.3.1.2.3.3.6.1",	//numConnectionAttempts					[0]
					".1.3.6.1.4.1.5586.3.1.2.3.3.6.5",	//numAcceptablePreSessionDisconnects	[1]
					".1.3.6.1.4.1.5586.3.1.2.3.3.6.6",	//numUnacceptablePreSessionDisconnects	[2]
					".1.3.6.1.4.1.5586.3.1.2.3.3.6.7",	//numAcceptablePostSessionDisconnects	[3]
			   );
			   
	for ($a=0; $a<count($iar_oid); $a++) {
		$iarValues[$a]=format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, $iar_oid[$a]));
	}
	
	//((Connection Attempts Unacceptable Pre-session Disconnects) / ConnectionAttempts) * 100
	if (($iarValues[0] - $iarValues[2]) > 0) {
		$accessRate = ($iarValues[0]-$iarValues[2])/$iarValues[0]*100;
	} else {
		$accessRate = 0;
	}
	
	//Acceptable Post-Session Disconnects / (Connection Attempts  Acceptable Presession
	//Disconnects  Unacceptable Pre-session Disconnects) *100
	if (($iarValues[0] - $iarValues[1] - $iarValues[2]) > 0) {
		$retainRate = $iarValues[3]/($iarValues[0] - $iarValues[1] - $iarValues[2])*100;
	} else {
		$retainRate = 0;
	}
	
	return sprintf("accessRate:%0.2f retainRate:%0.2f", $accessRate, $retainRate);
}

function inc_cn_stats($inc_ipaddr){
	
	/*
	coreNetworkMisc .1.3.6.1.4.1.5586.3.1.2.3.4.	1	//cnDownlinkDataRate 	(kbps)	[0]
													2	//cnDownlinkPacketRate	(pktps)	[1]
													3	//l2tpTotalActiveSessions		[2]
													4	//cnUplinkDataRate		(kbps)	[3]
													5	//cnUplinkPacketRate	(pktps)	[4]
	*/
	$ics_oidw = ".1.3.6.1.4.1.5586.3.1.2.3.4";		
	$icsValues = get_ipw_snmpwalk($inc_ipaddr, $ics_oidw);
	
	return sprintf("dlRate:%u ulRate:%u ue:%u", format_snmp_string($icsValues[0]), format_snmp_string($icsValues[3]), format_snmp_string($icsValues[2]));
}

function inc_handover_stats($inc_ipaddr){
	
	/*
	handoverStatistics 1.3.6.1.4.1.5586.3.4.2.10.1		1	//handoverTotalInterIncReselections			[0]
														2	//handoverTotalFailedInterIncReselections	[1]
														3	//handoverNumberActiveUesOnRemoteSgsn		[2]
														4	//handoverNumberActiveUesOnRemoteRnc		[3]
	*/
	$ihs_oidw = "1.3.6.1.4.1.5586.3.4.2.10.1";
	$ihsValues = get_ipw_snmpwalk($inc_ipaddr, $ihs_oidw);
	
	if (format_snmp_string($ihsValues[0]) > 0) {
		$ihs_intReselRate = (format_snmp_string($ihsValues[0]) - format_snmp_string($ihsValues[1]))/
							(format_snmp_string($ihsValues[0]))*100;
	} else {
		$ihs_intReselRate = 100;
	}

	return sprintf("interReselRate:%0.2f numAUEremSGSN:%u numAUEremRNC:%u", $ihs_intReselRate, format_snmp_string($ihsValues[2]), format_snmp_string($ihsValues[3]));
}

function inc_cpu_stats($inc_ipaddr){
	
	/*
	inc cpu stats oid .1.3.6.1.4.1.5586.3.1.2.5.6.1.1	4	cni (5-cni cpx1205)
														3	rni3
														2	rni2
														1 	rni1
	*/
	$icpu_oidw = ".1.3.6.1.4.1.5586.3.1.2.5.6.1.1";
	$icpuValues = get_ipw_snmpwalk($inc_ipaddr, $icpu_oidw);
	$icpuHardType = format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, ".1.3.6.1.4.1.5586.2.1.1.2.1.13"));
	
	$icpuValues = str_replace("::", "", $icpuValues);
	$icpuValues = str_replace("enterprises.", "", $icpuValues);
	$icpuValues = str_replace("SNMPv2-SMI", "", $icpuValues);
	$icpuValues = str_replace(".", ":", $icpuValues);
	$icpuValues = str_replace("=", ":", $icpuValues);
	$icpuValues = str_replace(" ", "", $icpuValues);
	
	$icpuSlot[5]=0; $icpuSlot[4]=0; $icpuSlot[3]=0; $icpuSlot[2]=0; $icpuSlot[1]=0;
	for ($a=0; $a<count($icpuValues); $a++) {
		$tmpSlotarr = split(":", $icpuValues[$a]);
		$icpuSlot[$tmpSlotarr[8]] = $tmpSlotarr[10];
	}
	
	if ($icpuHardType="CPX1204") {
		$cnicpu = $icpuSlot[4];
	} else {
		$cnicpu = $icpuSlot[5];
	}
	
	return sprintf("cni:%u rni3:%u rni2:%u rni1:%u", $cnicpu, $icpuSlot[3], $icpuSlot[2], $icpuSlot[1]);
	
}

function inc_l2tp_stats($inc_ipaddr){
	/*
	coreNetworkMisc .1.3.6.1.4.1.5586.3.1.2.3.4.	1	//cnDownlinkDataRate 	(kbps)	[0]
													2	//cnDownlinkPacketRate	(pktps)	[1]
													3	//l2tpTotalActiveSessions		[2]
													4	//cnUplinkDataRate		(kbps)	[3]
													5	//cnUplinkPacketRate	(pktps)	[4]
	raStatsEntry	1.3.6.1.4.1.5586.3.2.2.2.5.3.1.
						4	raNumberOfUEsConnected
	*/
	$ils_oid = ".1.3.6.1.4.1.5586.3.1.2.3.4.3";		
	$ilsUECell_oid= ".1.3.6.1.4.1.5586.3.2.2.2.5.3.1.4";
	$ilstotUECelloid = get_ipw_totNumUEoid($inc_ipaddr);
	$ilsUECellValues = get_trim_array(get_ipw_snmpwalk($inc_ipaddr, $ilsUECell_oid));
	
	$ils_l2tp = format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, $ils_oid));
	
	$numUEonCell[1]=0; $numUEonCell[2]=0; $numUEonCell[3]=0; $numUEonCell[4]=0; $numUEonCell[5]=0; 
	$numUEonCell[6]=0; $numUEonCell[7]=0; $numUEonCell[8]=0; $numUEonCell[9]=0; $numUEonCell[10]=0; 
	$numUEonCell[11]=0; $numUEonCell[12]=0;
	for ($a=0; $a < count($ilsUECellValues); $a++) { 
		$tmpValues = split(":", $ilsUECellValues[$a]);
		for ($b=0; $b < count($tmpValues); $b++) {
			if ($tmpValues[10]==$ilstotUECelloid ) { 
				$numUEonCell[$tmpValues[9]] = $tmpValues[12];
			}
		}
	}
	
	return sprintf("totL2TP:%u Cell1:%u Cell2:%u Cell3:%u Cell4:%u Cell5:%u Cell6:%u Cell7:%u Cell8:%u Cell9:%u Cell10:%u Cell11:%u Cell12:%u", $ils_l2tp, $numUEonCell[1], $numUEonCell[2], $numUEonCell[3], $numUEonCell[4], $numUEonCell[5], $numUEonCell[6], $numUEonCell[7], $numUEonCell[8], $numUEonCell[9], $numUEonCell[10], $numUEonCell[11], $numUEonCell[12]);
}


// nb stats functions ----------------------------------------------------------------------------------------------------
function nb_connresel_rate($inc_ipaddr, $nb_cellid) {
	
	if (empty($nb_cellid)) {
		$nb_cellid=1;
	}
	
	$ncr_oid = array(".1.3.6.1.4.1.5586.3.2.2.2.5.11.1.1.$nb_cellid", 			//rrcSuccConnAttp		[0]
						 ".1.3.6.1.4.1.5586.3.2.2.2.5.11.1.2.$nb_cellid", 	//rrcFailConnAttp		[1]
						".1.3.6.1.4.1.5586.3.2.2.2.5.10.1.1.$nb_cellid", 	//rrcSuccCelReselInCell	[2]
						".1.3.6.1.4.1.5586.3.2.2.2.5.10.1.2.$nb_cellid", 	//rrcFailCelReselInCell	[3]
						".1.3.6.1.4.1.5586.3.2.2.2.5.10.1.3.$nb_cellid", 	//rrcSuccCelReselOutCell[4]
						".1.3.6.1.4.1.5586.3.2.2.2.5.10.1.4.$nb_cellid"); 	//rrcFailCelReselOutCell[5]
						
	for ($a=0; $a<count($ncr_oid); $a++) {
		$ncrValues[$a]=format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, $ncr_oid[$a]));
	}
	
	if (($ncrValues[0]+$ncrValues[1])>0) {
		$perConnSuccRate=($ncrValues[0]-$ncrValues[1])/($ncrValues[0]+$ncrValues[1])*100;
	}else{
		$perConnSuccRate = 0;
	}
	
	if (($ncrValues[2]>0) || ($ncrValues[3]>0) || ($ncrValues[4]>0) || ($ncrValues[5]>0)) {
		$perReselSuccRate = ($ncrValues[2]+$ncrValues[4])/($ncrValues[2]+$ncrValues[3]+$ncrValues[4]+$ncrValues[5])*100;
	} else {
		$perReselSuccRate = 0;
	}

	return sprintf("connSuccRate:%0.2f reselSuccRate:%0.2f", $perConnSuccRate, $perReselSuccRate);
}

function nb_throughput_stats($inc_ipaddr, $nb_cellid){
	
	/*
	raMiscStatsEntry 1.3.6.1.4.1.5586.3.2.2.2.5.7.1.
						1	raUlAverageTotalThroughputAllocated
						2	raDlAverageTotalThroughputAllocated
						5	raUlAverageTotalThroughputUsed
						6	raDlAverageTotalThroughputUsed
	raStatsEntry	1.3.6.1.4.1.5586.3.2.2.2.5.3.1.
						4	raNumberOfUEsConnected	
	*/
	$nts_numueoid = get_ipw_totNumUEoid($inc_ipaddr);
	$nts_oid = array(".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.2.$nb_cellid",	//raDlAverageTotalThroughputAllocated	[0]
					".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.6.$nb_cellid",	//raDlAverageTotalThroughputUsed		[1]
					".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.1.$nb_cellid",	//raUlAverageTotalThroughputAllocated	[2]
					".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.5.$nb_cellid",	//raUlAverageTotalThroughputUsed		[3]
					".1.3.6.1.4.1.5586.3.2.2.2.5.3.1.4.$nb_cellid.$nts_numueoid" );//raNumberOfUEsConnected	[4]
	
	for ($ntsarr=0; $ntsarr < count($nts_oid); $ntsarr++) {
		$ntsValues[$ntsarr]=format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, $nts_oid[$ntsarr]));
	}
	
	if ($ntsValues[4] > 0) {
		$dlavgthougputperconnuser = $ntsValues[1]/$ntsValues[4]*1000;
		$ulavgthougputperconnuser = $ntsValues[3]/$ntsValues[4]*1000;
	} else {
		$dlavgthougputperconnuser = 0;
		$ulavgthougputperconnuser = 0;
	}
	
	return 	sprintf("raDlAvgThrougputAllo:%u raDlAvgThrougputUsed:%u raUlAvgThrougputAllo:%u raUlAvgThrougputUsed:%u DLAvgThrougputPerUser:%u ULAvgThrougputPerUser:%u raNumUEConnected:%u", $ntsValues[0]*1000, $ntsValues[1]*1000, $ntsValues[2]*1000, $ntsValues[3]*1000, $dlavgthougputperconnuser, $ulavgthougputperconnuser, $ntsValues[4]);		
	
}

function nb_dlTFCS_stats($inc_ipaddr, $nb_cellid){
	
	/*raDownlinkTFCSStatusEntry	
		OID:1.3.6.1.4.1.5586.3.2.2.2.31.1.
				2.	//raDownlinkTFCSResourceSpaceUsed
				#.	//cellid
				1=TFCS1, 2=TFCS2, 3=TFCS3, 4=TFCS4, 5=TFCS5, 6=TFCS6, 7=TFCS7, 8=TFCS8
				9 =CellEdge2
				10=CellEdge1				
	*/
	
	$ndTFCS_oidw = ".1.3.6.1.4.1.5586.3.2.2.2.31.1.2.$nb_cellid";
	$ndTFCSValues= get_ipw_snmpwalk($inc_ipaddr, $ndTFCS_oidw);
	
	$ndTFCSValues = str_replace("::", "", $ndTFCSValues);
	$ndTFCSValues = str_replace("enterprises.", "", $ndTFCSValues);
	$ndTFCSValues = str_replace("SNMPv2-SMI", "", $ndTFCSValues);
	$ndTFCSValues = str_replace(".", ":", $ndTFCSValues);
	$ndTFCSValues = str_replace("=", ":", $ndTFCSValues);
	$ndTFCSValues = str_replace(" ", "", $ndTFCSValues);
	for ($a=0; $a<count($ndTFCSValues); $a++) {
		$tmpTFCSarr = split(":", $ndTFCSValues[$a]);
		$ndTFCS[$tmpTFCSarr[9]] = $tmpTFCSarr[11];
	}
	return sprintf("dlcedge2:%u dlcedge1:%u tfcs1:%u tfcs2:%u tfcs3:%u tfcs4:%u tfcs5:%u tfcs6:%u tfcs7:%u tfcs8:%u", $ndTFCS[9], $ndTFCS[10], $ndTFCS[1], $ndTFCS[2], $ndTFCS[3], $ndTFCS[4], $ndTFCS[5], $ndTFCS[6], $ndTFCS[7], $ndTFCS[8] );
}

function nb_ulTFCS_stats($inc_ipaddr, $nb_cellid){
	
	/*raUplinkTFCSStatusEntry 
		OID:	1.3.6.1.4.1.5586.3.2.2.2.30.1.
					2.	//raUplinkTFCSResourceSpaceUsed
					#.	//cellid
					1=TFCS1, 2=TFCS2, 3=TFCS3, 4=TFCS4
	*/
	$nuTFCS_oidw = ".1.3.6.1.4.1.5586.3.2.2.2.30.1.2.$nb_cellid";
	$nuTFCSValues= get_ipw_snmpwalk($inc_ipaddr, $nuTFCS_oidw);
	
	$nuTFCSValues = str_replace("::", "", $nuTFCSValues);
	$nuTFCSValues = str_replace("enterprises.", "", $nuTFCSValues);
	$nuTFCSValues = str_replace("SNMPv2-SMI", "", $nuTFCSValues);
	$nuTFCSValues = str_replace(".", ":", $nuTFCSValues);
	$nuTFCSValues = str_replace("=", ":", $nuTFCSValues);
	$nuTFCSValues = str_replace(" ", "", $nuTFCSValues);
	
	for ($a=0; $a<count($nuTFCSValues); $a++) {
		$tmpTFCSarr = split(":", $nuTFCSValues[$a]);
		$nuTFCS[$tmpTFCSarr[9]] = $tmpTFCSarr[11];
	}
	
	return sprintf("tfcs1:%u tfcs2:%u tfcs3:%u tfcs4:%u", $nuTFCS[1], $nuTFCS[2], $nuTFCS[3], $nuTFCS[4] );

}

function nb_queue_stats($inc_ipaddr, $nb_cellid){
	
	/*
	raMiscStatsEntry 1.3.6.1.4.1.5586.3.2.2.2.5.7.1.
						15	raUlTotalQueuedUEs	
						16	raDlTotalQueuedUEs
	raStatsEntry	1.3.6.1.4.1.5586.3.2.2.2.5.3.1.
						4	raNumberOfUEsConnected	
	*/
	$nqs_numueoid = get_ipw_totNumUEoid($inc_ipaddr);
	$nqs_oid = array(".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.16.$nb_cellid",				//raDlTotalQueuedUEs	[0]	
					".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.15.$nb_cellid",				//raUlTotalQueuedUEs	[1]
					".1.3.6.1.4.1.5586.3.2.2.2.5.3.1.4.$nb_cellid.$nqs_numueoid" );	//raNumberOfUEsConnected[2]
	
	for ($nqsarr=0; $nqsarr<count($nqs_oid); $nqsarr++) {
		$nqsValues[$nqsarr] = format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, $nqs_oid[$nqsarr]));
	}
	
	return sprintf("dlQueuedUE:%0.2f ulQueuedUE:%0.2f numUE:%u", $nqsValues[0], $nqsValues[1], $nqsValues[2]);
}

function nb_allocinterval_stats($inc_ipaddr, $nb_cellid){
	
	/*raToSLinkStatsEntry
		OID:	1.3.6.1.4.1.5586.3.2.2.2.5.8.1.
					11.	//raToSLinkAveAllocationInterval
					12.	//raToSLinkMaxAllocationInterval
				#.	//cellid
				#.	//TOS(1-4) or (1-8)
				1/2	//UL/DL
	raStatsEntry	1.3.6.1.4.1.5586.3.2.2.2.5.3.1.
						4	raNumberOfUEsConnected	
	raConfigEntry	1.3.6.1.4.1.5586.3.2.2.2.16.1
						33.cellid	raEnableTiers 1=(1-4), 2=(1-8)
	*/
	$nas_numueoid = get_ipw_totNumUEoid($inc_ipaddr);
	$nas_oid = array(".1.3.6.1.4.1.5586.3.2.2.2.5.8.1.11.$nb_cellid",				//raToSLinkAveAllocationInterval	[arr]
					".1.3.6.1.4.1.5586.3.2.2.2.5.8.1.12.$nb_cellid",				//raToSLinkMaxAllocationInterval	[arr]
					".1.3.6.1.4.1.5586.3.2.2.2.5.3.1.4.$nb_cellid.$nas_numueoid",	//raNumberOfUEsConnected			[int]
					".1.3.6.1.4.1.5586.3.2.2.2.16.1.33.$nb_cellid");				//raEnableTiers						[int]
	
	//$nasValues[0] = get_ipw_snmpwalk($inc_ipaddr, $nas_oid[0]);
	for ($a=0; $a < count($nas_oid); $a++){
		$nasValues[$a] = get_ipw_snmpwalk($inc_ipaddr, $nas_oid[$a]);
	}
	
	$dlMaxAllocInt = 0; $ulMaxAllocInt = 0;
	$sumdlAveAllocInt = 0; $sumulAveAllocInt = 0;
	$sumdlCount = 0; $sumulCount =0;
	
	for ($a=0; $a < 2; $a++) {
		$nasValues[$a] = str_replace("::", "", $nasValues[$a]);
		$nasValues[$a] = str_replace("enterprises.", "", $nasValues[$a]);
		$nasValues[$a] = str_replace("SNMPv2-SMI", "", $nasValues[$a]);
		$nasValues[$a] = str_replace(".", ":", $nasValues[$a]);
		$nasValues[$a] = str_replace("=", ":", $nasValues[$a]);
		$nasValues[$a] = str_replace(" ", "", $nasValues[$a]);
		
		for ($b=0; $b<count($nasValues[$a]); $b++) {
			$tmpValues = split(":", $nasValues[$a][$b]);
			if ($tmpValues[8]==12 && $tmpValues[11]==2) {	//raToSLinkMaxAllocationInterval -DL
				if ($tmpValues[13] > $dlMaxAllocInt) { //DL
					$dlMaxAllocInt = $tmpValues[13];
				}
			}
			if ($tmpValues[8]==12 && $tmpValues[11]==1) {  //raToSLinkMaxAllocationInterval -UL
				if ($tmpValues[13] > $ulMaxAllocInt) { //DL
					$ulMaxAllocInt = $tmpValues[13];
				}
			}
			if ($tmpValues[8]==11 && $tmpValues[11]==2) { //raToSLinkAveAllocationInterval - DL
				if ($tmpValues[13]>0) {
					$sumdlAveAllocInt = $sumdlAveAllocInt + $tmpValues[13];
					$sumdlCount = $sumdlCount + 1;
				}
			}
			if ($tmpValues[8]==11 && $tmpValues[11]==1) { //raToSLinkAveAllocationInterval - UL
				if ($tmpValues[13]>0) {
					$sumulAveAllocInt = $sumulAveAllocInt + $tmpValues[13];
					$sumulCount = $sumulCount + 1;
				}
			}
		}
	}
	
	$numUE = format_snmp_string($nasValues[2]);

	$dlAveAllocInt = 0;
	if ($sumdlAveAllocInt > 0) {
		$dlAveAllocInt = $sumdlAveAllocInt/$sumdlCount;
	} 
	$ulAveAllocInt = 0;
	if ($sumulAveAllocInt > 0) {
		$ulAveAllocInt = $sumulAveAllocInt/$sumulCount;
	}	
	
	return sprintf("dlAveAllocInt:%u dlMaxAllocInt:%u ulAveAllocInt:%u ulMaxAllocInt:%u numUE:%u", $dlAveAllocInt, $dlMaxAllocInt, $ulAveAllocInt, $ulMaxAllocInt, $numUE);
}

function nb_dlresource_stats($inc_ipaddr, $nb_cellid){
	
	/*
	raMiscStatsEntry 1.3.6.1.4.1.5586.3.2.2.2.5.7.1.
						17	raDlPseudoDCHResourceUnitsUsed
						8	raDlPredictedResourceUnitsUsed
						10	raDlNormalResourceUnitsUsed
						18	raDlCellEdgeResourceUnitsUsed
						4	raDlResourceUnitsUsed //total
	raStatsEntry	1.3.6.1.4.1.5586.3.2.2.2.5.3.1.
						4	raNumberOfUEsConnected	
	*/
	
	$ndlrs_oid = array(".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.8.$nb_cellid",	//raDlPredictedResourceUnitsUsed	[0]
					".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.10.$nb_cellid",	//raDlNormalResourceUnitsUsed		[1]
					".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.17.$nb_cellid",	//raDlPseudoDCHResourceUnitsUsed	[2]
					".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.18.$nb_cellid",	//raDlCellEdgeResourceUnitsUsed		[3]
					".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.4.$nb_cellid"); 	//raDlResourceUnitsUsed(total)		[4]
	
	for ($a=0; $a < count($ndlrs_oid); $a++) {
		$ndlrsValues[$a] = format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, $ndlrs_oid[$a]));
	}
	
	return sprintf("PredictedRUsed:%u NormalRUsed:%u PseudoDCHRUsed:%u CellEdgeRUsed:%u total:%u", $ndlrsValues[0], $ndlrsValues[1], $ndlrsValues[2], $ndlrsValues[3], $ndlrsValues[4]);
}

function nb_ulresource_stats($inc_ipaddr, $nb_cellid){
	
	/*
	raMiscStatsEntry 1.3.6.1.4.1.5586.3.2.2.2.5.7.1.
						12	raUlPseudoDCHResourceUnitsUsed
						11	raUlPriorityPredictedResourceUnitsUsed
						7	raUlPredictedResourceUnitsUsed
						9	raUlNormalResourceUnitsUsed
						3	raUlResourceUnitsUsed //total
	raStatsEntry	1.3.6.1.4.1.5586.3.2.2.2.5.3.1.
						4	raNumberOfUEsConnected	
	*/
	
	$nulrs_oid = array(".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.7.$nb_cellid",	//raUlPredictedResourceUnitsUsed		[0]
					".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.9.$nb_cellid",		//raUlNormalResourceUnitsUsed			[1]
					".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.11.$nb_cellid",	//raUlPriorityPredictedResourceUnitsUsed[2]
					".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.12.$nb_cellid",	//raUlPseudoDCHResourceUnitsUsed		[3]
					".1.3.6.1.4.1.5586.3.2.2.2.5.7.1.3.$nb_cellid"		//raUlResourceUnitsUsed					[4]
					);
	
	for ($a=0; $a < count($nulrs_oid); $a++) {
		$nulrsValues[$a] = format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, $nulrs_oid[$a]));
	}
	
	return sprintf("PredictedRUsed:%u NormalRUsed:%u PriorityRUsed:%u PseudoDCHRUsed:%u total:%u", $nulrsValues[0], $nulrsValues[1], $nulrsValues[2], $nulrsValues[3], $nulrsValues[4]);
}

function nb_ibts_stats($inc_ipaddr, $nb_cellid){
	
	/*
	iSCPMeasurementEntry 
		OID = .1.3.6.1.4.1.5586.3.2.2.2.5.6.1.2.
				#.	Cellid
				TS	timeslot number						
		dBm. Offset is -116dBm
		
	nodeBNBAPEntry
		OID:	.1.3.6.1.4.1.5586.3.2.2.2.3.1.1.
					.10. //nodeBNBAPPUSCHStartTs
					.11. //nodeBNBAPPUSCHStopTs
					.12. //nodeBNBAPPRACHTs
					.15. //nodeBNBAPExtendedCell
					# Cellid
	*/
	
	$nuIBTS_oidw = ".1.3.6.1.4.1.5586.3.2.2.2.5.6.1.2.$nb_cellid";
	$nuIBTSprachts_oid = ".1.3.6.1.4.1.5586.3.2.2.2.3.1.1.12.$nb_cellid";
	$nuIBTSpushstart_oid = ".1.3.6.1.4.1.5586.3.2.2.2.3.1.1.10.$nb_cellid";
	$nuIBTSpushstop_oid = ".1.3.6.1.4.1.5586.3.2.2.2.3.1.1.11.$nb_cellid";
	//$nuIBTSextcell_oid = ".1.3.6.1.4.1.5586.3.2.2.2.3.1.1.15.$nb_cellid";
	
	$nuIBTSValues= get_ipw_snmpwalk($inc_ipaddr, $nuIBTS_oidw);
	$nuIBTSprachTS= format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, $nuIBTSprachts_oid));
	$nuIBTSpushstart = format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, $nuIBTSpushstart_oid));
	$nuIBTSpushstop = format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, $nuIBTSpushstop_oid));
	//$nuIBTSextcell = format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, $nuIBTSextcell_oid));
	
	$nuIBTStsvalue = 0; $nuIBTSts[11]=0; $nuIBTSts[12]=0; $nuIBTSts[13]=0; $nuIBTSts[14]=0;
	if (is_int($nuIBTSpushstart) && is_int($nuIBTSpushstop)) {
		for ($a=$nuIBTSpushstart; $a <= $nuIBTSpushstop; $a++) {
			$nuIBTStsvalue = format_snmp_string($nuIBTSValues[$a]);
			if ($nuIBTStsvalue > 0) {
				$nuIBTSts[$a] = ($nuIBTStsvalue - 116);
			}
		}
	} else {
		for ($a=0; $a < count($nuIBTSValues); $a++) {
			$nuIBTStsvalue = format_snmp_string($nuIBTSValues[$a]);
			if ($nuIBTStsvalue > 0) {
				$nuIBTSts[$a] = ($nuIBTStsvalue - 116);
			}
		}
	}	
	
	$nuIBTStsRACH = 0;
	$nuIBTStsRACHval = format_snmp_string($nuIBTSValues[$nuIBTSprachTS]);
	if ($nuIBTStsRACHval > 0) {
		$nuIBTStsRACH = ($nuIBTStsRACHval - 116);
	}
	
	return sprintf("tsRACH:%d ts11:%d ts12:%d ts13:%d ts14:%d", $nuIBTStsRACH, $nuIBTSts[11], $nuIBTSts[12], $nuIBTSts[13], $nuIBTSts[14]);
}

function nb_numue_stats($inc_ipaddr, $nb_cellid){
	/*
	raStatsEntry	1.3.6.1.4.1.5586.3.2.2.2.5.3.1.
						4	raNumberOfUEsConnected	
	*/
	
	$nues_oidw = ".1.3.6.1.4.1.5586.3.2.2.2.5.3.1.4.$nb_cellid";
	$nuesValues = get_ipw_snmpwalk($inc_ipaddr, $nues_oidw);
	$nues_numueoid = get_ipw_totNumUEoid($inc_ipaddr);
	
	$nues_tiers[1]=0; $nues_tiers[2]=0; $nues_tiers[3]=0; $nues_tiers[4]=0;
	$nues_tiers[5]=0; $nues_tiers[6]=0; $nues_tiers[7]=0; $nues_tiers[8]=0;
	$nues_total = format_snmp_string($nuesValues[4]);
	for ($a=0; $a < count($nuesValues); $a++) {
		$nues_tiers[$a+1]= format_snmp_string($nuesValues[$a]);
	}
	
	$nues_total = 0;
	if (count($nuesValues) > 5) {
		$nues_total = format_snmp_string($nuesValues[8]);
	} else {
		$nues_total = format_snmp_string($nuesValues[4]);
	}
	
	return sprintf("tier1:%u tier2:%u tier3:%u tier4:%u tier5:%u tier6:%u tier7:%u tier8:%u total:%u", $nues_tiers[1], $nues_tiers[2], $nues_tiers[3], $nues_tiers[4], $nues_tiers[5], $nues_tiers[6], $nues_tiers[7], $nues_tiers[8], $nues_total);
}

function nb_ulavebitrate($inc_ipaddr, $nb_cellid){
	/*
	raToSLinkStatsTierOfService 
		OID:	1.3.6.1.4.1.5586.3.2.2.2.5.8.1.
					.8		//raToSLinkAverageBitRate
					.#		// Cellid
					.tiers	//1-8/1-4 tier
					.1/2	// 1 uplink 2 downlink		
	*/
	
	$nabr_oidw = ".1.3.6.1.4.1.5586.3.2.2.2.5.8.1.8.$nb_cellid";
	$nabrValues = get_trim_array(get_ipw_snmpwalk($inc_ipaddr, $nabr_oidw));
	
	for ($a=0; $a < count($nabrValues); $a++) {
		$tmpArr = split(":", $nabrValues[$a]);
		for ($b=0; $b < count($tmpArr); $b++) {
			if ($tmpArr[11]==1) { //uplink tiers
				$ulTiers[$tmpArr[10]] = $tmpArr[13];
			} 
		}
	}
	
	return sprintf("ultier1:%u ultier2:%u ultier3:%u ultier4:%u ultier5:%u ultier6:%u ultier7:%u ultier8:%u", $ulTiers[1]*1000, $ulTiers[2]*1000, $ulTiers[3]*1000, $ulTiers[4]*1000, $ulTiers[5]*1000, $ulTiers[6]*1000, $ulTiers[7]*1000, $ulTiers[8]*1000);
	
}

function nb_dlavebitrate($inc_ipaddr, $nb_cellid){
	/*
	raToSLinkStatsTierOfService 
		OID:	1.3.6.1.4.1.5586.3.2.2.2.5.8.1.
					.8		//raToSLinkAverageBitRate
					.#		// Cellid
					.tiers	//1-8/1-4 tier
					.1/2	// 1 uplink 2 downlink		
	*/
	
	$nabr_oidw = ".1.3.6.1.4.1.5586.3.2.2.2.5.8.1.8.$nb_cellid";
	$nabrValues = get_trim_array(get_ipw_snmpwalk($inc_ipaddr, $nabr_oidw));
	
	for ($a=0; $a < count($nabrValues); $a++) {
		$tmpArr = split(":", $nabrValues[$a]);
		for ($b=0; $b < count($tmpArr); $b++) {
			if ($tmpArr[11]==2) { //downlink tiers
				$dlTiers[$tmpArr[10]] = $tmpArr[13];
			}
		}
	}
	
	return sprintf("dltier1:%d dltier2:%d dltier3:%d dltier4:%d dltier5:%d dltier6:%d dltier7:%d dltier8:%d", $dlTiers[1]*1000, $dlTiers[2]*1000, $dlTiers[3]*1000, $dlTiers[4]*1000, $dlTiers[5]*1000, $dlTiers[6]*1000, $dlTiers[7]*1000, $dlTiers[8]*1000);
	
}


// generic functions --------------------------------------------------------------------------------------------------
function get_ipw_snmpwalk($inc_ipaddr, $oid) {
	$cmd = "snmpwalk -v 3 -u user1 -n DefaultContextName " . $inc_ipaddr . " " . $oid;
	exec($cmd, $arr);
	if (count($arr)>1) {
		$out_array = $arr;
		return $out_array;
	} else {
		$out_string=$arr[0];
		return $out_string;
	}
}

function get_ipw_totNumUEoid($inc_ipaddr){
	//bootloaderslot_oid: 1.3.6.1.4.1.5586.2.1.4.2.3.1.7.1
	$currboot = format_snmp_string(get_ipw_snmpwalk($inc_ipaddr, ".1.3.6.1.4.1.5586.2.1.4.2.3.1.7.1"));
	
	$currboot = str_replace(".", ":", $currboot);
	$currboot = str_replace("blo", "", $currboot);
	$bootfirm = split(":", $currboot);
	
	if (($bootfirm[0]>=5) && ($bootfirm[1]>=2) && ($bootfirm[2]>=65)) {
		$numUE_oid = "100";
	} elseif (($bootfirm[0]>=5) && ($bootfirm[1]>=4) && ($bootfirm[2]>=1)) {
		$numUE_oid = "100";
	} else {
		$numUE_oid = "5";
	}
	return $numUE_oid;
}

function get_trim_array($arrValues){
	$arrValues = str_replace("::", "", $arrValues);
	$arrValues = str_replace("enterprises.", "", $arrValues);
	$arrValues = str_replace("SNMPv2-SMI", "", $arrValues);
	$arrValues = str_replace(".", ":", $arrValues);
	$arrValues = str_replace("=", ":", $arrValues);
	$arrValues = str_replace(" ", "", $arrValues);	
	return $arrValues;
}

function format_snmp_string($string) {
	/* strip off all leading junk (the oid and stuff) */
	$string = trim(ereg_replace(".*= ?", "", $string));

	/* remove ALL quotes */
	$string = str_replace("\"", "", $string);
	$string = str_replace("'", "", $string);
	$string = str_replace(">", "", $string);
	$string = str_replace("<", "", $string);
	$string = str_replace("\\", "", $string);

	/* Account for invalid type messages */
	if (substr_count($string, "Wrong Type")) {
		$string = strrev($string);
		if ($position = strpos($string, ":")) {
			$string = trim(strrev(substr($string, 0, $position)));
		}else{
			$string = trim(strrev($string));
		}
	}

	/* Remove invalid chars */
	$k = strlen($string);
	for ($i=0; $i < $k; $i++) {
		if ((ord($string[$i]) <= 31) || (ord($string[$i]) >= 127)) {
			$string[$i] = " ";
		}
	}
	$string = trim($string);

	if ((substr_count($string, "Hex-STRING:")) ||
		(substr_count($string, "Hex:"))) {
		/* strip of the 'Hex-STRING:' */
		$string = eregi_replace("Hex-STRING: ?", "", $string);
		$string = eregi_replace("Hex: ?", "", $string);

		$string_array = split(" ", $string);

		/* loop through each string character and make ascii */
		$string = "";
		$hexval = "";
		$ishex  = false;
		for ($i=0;($i<sizeof($string_array));$i++) {
			if (strlen($string_array[$i])) {
				$string .= chr(hexdec($string_array[$i]));

				$hexval .= str_pad($string_array[$i], 2, "0", STR_PAD_LEFT);

				if (($i+1) < count($string_array)) {
					$hexval .= ":";
				}

				if ((hexdec($string_array[$i]) <= 31) || (hexdec($string_array[$i]) >= 127)) {
					if ((($i+1) == sizeof($string_array)) && ($string_array[$i] == 0)) {
						/* do nothing */
					}else{
						$ishex = true;
					}
				}
			}
		}

		if ($ishex) $string = $hexval;
	}elseif (preg_match("/(hex:\?)?([a-fA-F0-9]{1,2}(:|\s)){5}/", $string)) {
		$octet = "";

		/* strip of the 'hex:' */
		$string = eregi_replace("hex: ?", "", $string);

		/* split the hex on the delimiter */
		$octets = preg_split("/\s|:/", $string);

		/* loop through each octet and format it accordingly */
		for ($i=0;($i<count($octets));$i++) {
			$octet .= str_pad($octets[$i], 2, "0", STR_PAD_LEFT);

			if (($i+1) < count($octets)) {
				$octet .= ":";
			}
		}

		/* copy the final result and make it upper case */
		$string = strtoupper($octet);
	}elseif (preg_match("/Timeticks:\s\((\d+)\)\s/", $string, $matches)) {
		$string = $matches[1];
	}

	$string = eregi_replace(REGEXP_SNMP_TRIM, "", $string);

	return trim($string);
}
?>